
#include "IMGTexture.h"

namespace IMG 
{

Texture::Texture(): references(0) // initialize references to 0
{
	mTexture = 0;
	mWidth = 0;
	mHeight = 0;
	mMaterial = 0;
	mCalculatedAvgColour = false;
	mRectangle2D = new Ogre::Rectangle2D(true);
	mRectangle2D->setBoundingBox(Ogre::AxisAlignedBox(-100000.0 * Ogre::Vector3::UNIT_SCALE, 100000.0 * Ogre::Vector3::UNIT_SCALE));
	mRectangle2D->setRenderQueueGroup(Ogre::RENDER_QUEUE_OVERLAY);
}

Texture::~Texture()
{
	// Gametools -- BUG: 06/04/2006
	//if (mRectangle2D)
	//{
	//	delete mRectangle2D;
	//}

	//if (mMaterial)
	//{
	//	delete mMaterial;
	//}
}

Ogre::Rectangle2D* Texture::getRectangle2D()
{
	return mRectangle2D;
}

void Texture::setCorners(Ogre::Real left, Ogre::Real top, Ogre::Real right, Ogre::Real bottom)
{
	mRectangle2D->setCorners(left, top, right, bottom);
}

void Texture::bind(Ogre::String textureName)
{		
	if (!Ogre::TextureManager::getSingleton().getByName(textureName).isNull())
	{
		mTexture = (Ogre::Texture*) Ogre::TextureManager::getSingleton().getByName(textureName).getPointer();
		mWidth = mTexture->getSrcWidth();
		mHeight = mTexture->getSrcHeight();
	}
	else
	{
		Ogre::TexturePtr tPtr = Ogre::TextureManager::getSingleton().load(textureName, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);

		mTexture = (Ogre::Texture*) tPtr.getPointer();
		mWidth = mTexture->getSrcWidth();
		mHeight = mTexture->getSrcHeight();
	}
}

void Texture::setBackgroundColour(Ogre::ColourValue colour)
{
	mBackgroundColour = colour;
}

Ogre::ColourValue Texture::getBackgroundColour()
{
	return mBackgroundColour;
}

Ogre::ColourValue Texture::getAvgColour()
{
	if (mCalculatedAvgColour)
	{
		return mAvgColour;
	}
	else
	{
		calculateAvgColour();
		return mAvgColour;
	}
}

void Texture::calculateAvgColour()
{
	unsigned int cR = 0;
	unsigned int cG = 0;
	unsigned int cB = 0;
	unsigned int cA = 0;

	Ogre::Image img;
	img.load(mTexture->getName(), Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME); 
	unsigned char* data = static_cast<unsigned char*>(img.getData());

	unsigned int nColorpixels = 1;

	for (unsigned int i = 0; i < mTexture->getWidth() * mTexture->getHeight() * 4; i=i+4)
	{
		cA = (unsigned short int)data[i+3];
		if (cA != 0)
		{
			cB = cB + data[i+2];
			cG = cG + data[i+1];
			cR = cR + data[i];				

			nColorpixels++;
			//Ogre::LogManager::getSingleton().logMessage("(" + Ogre::StringConverter::toString(data[i]) + "," + Ogre::StringConverter::toString(data[i+1]) + "," + Ogre::StringConverter::toString(data[i+2]) + "," + Ogre::StringConverter::toString(data[i+3]) + ")");
		}					
	}

	cR = cR / nColorpixels;
	cG = cG / nColorpixels;
	cB = cB / nColorpixels;

	mAvgColour = Ogre::ColourValue(cR/255.0, cG/255.0, cB/255.0, 0.0);
	mCalculatedAvgColour = true;
}

void Texture::load(Ogre::String folderName, Ogre::String fileName)
{
	Ogre::TextureManager::getSingleton().load(fileName, Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
	mTexture = (Ogre::Texture*)Ogre::TextureManager::getSingleton().getByName(fileName).getPointer();
	mWidth = mTexture->getSrcWidth();
	mHeight = mTexture->getSrcHeight();
}

void Texture::save(Ogre::String folderName, Ogre::String filename)
{
    Ogre::HardwarePixelBufferSharedPtr readbuffer;
    readbuffer = mTexture->getBuffer(0, 0);
	unsigned char *tex = new unsigned char[readbuffer->getSizeInBytes()];
	Ogre::PixelBox *pBox = new Ogre::PixelBox(mTexture->getWidth(), mTexture->getHeight(), mTexture->getDepth(), mTexture->getFormat(), tex);
    readbuffer->blitToMemory((*pBox));
    
    Ogre::Image img;
    img = img.loadDynamicImage(tex, mTexture->getWidth(), mTexture->getHeight(), mTexture->getDepth(), mTexture->getFormat(), true, 1, 0);		
	img.resize(mWidth, mHeight);
    img.save(folderName + filename);
	delete pBox;
}

void Texture::rotate(Ogre::Real rRotation)
{
	//mTexture->setTextureRotate( Degree( -rRotation ) );
}

void Texture::setWidth(unsigned int width)
{
	mWidth = width;
}

unsigned int Texture::getWidth()
{
	return mWidth;
}

void Texture::setHeight(unsigned int height)
{
	mHeight = height;
}

unsigned int Texture::getHeight()
{
	return mHeight;
}

void Texture::setTextureName(Ogre::String textureName)
{
	mTextureName = textureName;
}

Ogre::String Texture::getTextureName()
{
	return mTextureName;
}

void Texture::setMaterial(Ogre::Material *material)
{
	mMaterial = material;
	mRectangle2D->setMaterial(mMaterial->getName());
}

Ogre::Material* Texture::getMaterial()
{
	return mMaterial;
}

}
